{$I UOpt.INC}				(* Standard Conditional-Defines *)
unit Childwin;

interface

uses Messages, Windows, Classes, Graphics, Forms, Controls, ExtCtrls, ComCtrls, SysUtils,
     ClipBrd, Dialogs, SubChild, Menus;

CONST
  CM_ParentCalled = WM_USER + $1000;
  cmPCNothing     = 0;
  cmPCChangeSizeX = cmPCNothing + 1;
  cmPCChangeSizeY = cmPCNothing + 2;
  cmPCNewFileName = cmPCNothing + 3;

TYPE
  TParentCalled = PROCEDURE( VAR Msg: TMessage ) OF OBJECT;

type
  TMDIChild = class( TForm )
    PaintBox1: TPaintBox;
    MDIChildMenu: TMainMenu;
    Option1: TMenuItem;
    ClipboardExportWMF: TMenuItem;
    SaveAsMetafileWMF: TMenuItem;
    SaveAsMetafileEMF: TMenuItem;
    N1: TMenuItem;
    ClipboardExportEMF: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1DblClick(Sender: TObject);
    procedure ClipboardExportWMFClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ClipboardExportEMFClick(Sender: TObject);
    procedure SaveAsMetafileEMFClick(Sender: TObject);
    procedure SaveAsMetafileWMFClick(Sender: TObject);
  private
    theSizeLog         : TPoint;
    theFileName        : TFileName;
    theSubForm         : TSubForm;
    theSizePix         : TPoint;
    theMultiplier      : TPoint;
    bMouseCaptured     : BOOLEAN;
    tpMouseCaptured    : TPoint;
    tpLastMousePosition : TPoint;
    PROCEDURE DisplayMouseCoordinates( aPoint: TPoint );
    PROCEDURE RepositionScrollBars( aPoint: TPoint );
  protected
    PROCEDURE CMParentCalled( VAR aMsg: TMessage ); MESSAGE CM_ParentCalled;

  public
    PROCEDURE CopyToMetafile( aPlotMode: TPlotMode );
    CONSTRUCTOR CreateExtra(AOwner: TComponent; aFileName: TFileName; aSizeLog: TPoint );
    PROCEDURE WndProc(var Message: TMessage); override;
  published

    { Public declarations }
  end;

(*----------------------------------------------------*)
implementation

USES
  Main; {wg. TypCasting zu MainForm}

{$R *.DFM}


(*----------------------------------------------------*)
CONSTRUCTOR TMDIChild.CreateExtra(AOwner: TComponent; aFileName: TFileName; aSizeLog: TPoint );

BEGIN
theMultiplier   := Point( 20, 20 );
theSizeLog	:= aSizeLog;
theFileName	:= aFileName;
INHERITED Create( AOwner );
Caption         := aFileName;
END;

(*----------------------------------------------------*)
PROCEDURE TMDIChild.WndProc(VAR Message: TMessage);

BEGIN
INHERITED;
CASE Message.Msg OF
  WM_Size : BEGIN
	    IF (theSubForm <> nil) THEN
	      BEGIN
	      IF Message.wParam = SIZEICONIC THEN
		theSubForm.WindowState := wsMinimized;
	      IF Message.wParam = SIZENORMAL THEN
		theSubForm.WindowState := wsNormal;
	      END;
	    END;

  END;
END;

(*----------------------------------------------------*)
procedure TMDIChild.FormCreate(Sender: TObject);

begin
ClientWidth	:= 300;
WITH theSizeLog DO
  ClientHeight	:= MulDiv(ClientWidth, X, Y );

theSubForm := TSubForm.CreateExtra( Self,
                                    PaintBox1,
                                    theFileName,
                                    theSizeLog,
                                    theMultiplier,
                                    Self.Handle );
theSubForm.Show;
end;


(*----------------------------------------------------*)
procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);

begin
IF theSubForm.CloseQuery THEN
  Action := caFree
 ELSE
  Action := caNone;
end;

(*----------------------------------------------------*)
procedure TMDIChild.FormShow(Sender: TObject);

begin
WITH theSizePix DO
  BEGIN
  X := ClientWidth;
  Y := ClientHeight;
  END;
end;

(*----------------------------------------------------*)
procedure TMDIChild.FormResize(Sender: TObject);

begin
WITH theSizePix DO
  BEGIN
  X := ClientWidth;
  Y := ClientHeight;
  END;
end;

(*----------------------------------------------------*)
procedure TMDIChild.PaintBox1Paint(Sender: TObject);

begin
WITH PaintBox1.Canvas DO
  BEGIN
  IF theSizeLog.Y > theSizeLog.X THEN
   ClientWidth     := MulDiv(ClientHeight, theSizeLog.X, theSizeLog.Y)
  ELSE
   ClientHeight    := MulDiv(ClientWidth, theSizeLog.Y, theSizeLog.X);
  SetMapMode( Handle, mm_IsoTropic );
  SetWindowExtEx( Handle, theSizeLog.X, theSizeLog.Y, nil );
  SetViewPortExtEx( Handle, theSizePix.X, -theSizePix.Y, nil );
  SetWindowOrgEx( Handle, 0, theSizeLog.Y, nil );
  END;
theSubForm.PlotIt( PaintBox1.Canvas, pmPaint, true, Point(1, 1) );
end;

(*------------------------------------------------------*)
PROCEDURE TMDIChild.CopyToMetafile( aPlotMode: TPlotMode );

VAR
  tMyMetafile		: TMetafile;
  tMyMetafileCanvas	: TMetafileCanvas;
  aFileName             : TFileName;
  aSaveDialog	        : TSaveDialog;


begin
tMyMetafile		:= TMetafile.Create;
tMyMetafile.mmWidth	:= theSizeLog.X*theMultiplier.X;
tMyMetafile.mmHeight	:= theSizeLog.Y*theMultiplier.Y;
CASE aPlotMode OF
  pmClipboardEMF, pmEMF:
    BEGIN
    tMyMetafile.Enhanced:= true;
    tMyMetafileCanvas	:= TMetafileCanvas.CreateWithComment( tMyMetafile, 0, Application.Title, theFileName );
    END;
  pmClipboardWMF, pmWMF:
    BEGIN
    tMyMetafile.Enhanced:= false;
    tMyMetafileCanvas	:= TMetafileCanvas.Create( tMyMetafile, 0 );
    END;
  END;

WITH tMyMetafileCanvas DO
  BEGIN
  SetMapMode( Handle, mm_HiMetric );
  SetWindowOrgEx( Handle, 0, tMyMetafile.mmHeight, nil );
  Font		:= PaintBox1.Font;
  Pen.Width	:= 0; {MulDiv( 2, 2540, 72 );}
  Pen.Color	:= clBlack;
  theSubForm.PlotIt( tMyMetafileCanvas, aPlotMode, true, theMultiplier );
  Free;
  END;

CASE aPlotMode OF
  pmClipboardEMF, pmClipboardWMF:
    BEGIN
    ClipBoard.Assign( tMyMetafile );
    END;
  pmEMF, pmWMF:
    BEGIN
    aFileName := theFileName;
    Delete( aFileName, Length(aFileName)-Length(ExtractFileExt(aFileName))+1, Length(ExtractFileExt(aFileName)));
    aSaveDialog	:=  TSaveDialog.Create( Self );
    WITH aSaveDialog DO
      BEGIN
      IF (aPlotMode=pmWMF) THEN DefaultExt := 'WMF' ELSE DefaultExt := 'EMF';
      Options	           := [ofPathMustExist,ofOverwritePrompt];
      Filter	           := 'Metafiles|*.'+DefaultExt;
      FileName	           := aFileName;
      InitialDir	   := ExtractFilePath( FileName );
      IF Execute THEN
        BEGIN
        tMyMetafile.SaveToFile( FileName );
        END;
      END;
    aSaveDialog.Free;
    END;
  END;
tMyMetafile.Free;
end;

(*----------------------------------------------------*)
procedure TMDIChild.ClipboardExportWMFClick(Sender: TObject);

begin
CopyToMetafile( pmClipboardWMF );
end;

(*----------------------------------------------------*)
procedure TMDIChild.ClipboardExportEMFClick(Sender: TObject);
begin
CopyToMetafile( pmClipboardEMF );
end;

(*----------------------------------------------------*)
procedure TMDIChild.SaveAsMetafileWMFClick(Sender: TObject);
begin
CopyToMetafile( pmWMF );
end;

(*----------------------------------------------------*)
procedure TMDIChild.SaveAsMetafileEMFClick(Sender: TObject);
begin
CopyToMetafile( pmEMF );
end;

(*----------------------------------------------------*)
PROCEDURE TMDIChild.DisplayMouseCoordinates( aPoint: TPoint );

BEGIN
tpLastMousePosition := aPoint;
WITH TMDIMainForm(Application.Mainform).StatusBar DO
  BEGIN
  WITH theSubForm.ScrollBox1 DO
    BEGIN
    Panels[1].Text := 'X:'+IntToStr( Round(Int(aPoint.X)/Int(PaintBox1.ClientWidth)*Int( HorzScrollBar.Range )));
    Panels[2].Text := 'Y:'+IntToStr( Round(Int(PaintBox1.ClientHeight-aPoint.Y)/Int(PaintBox1.ClientHeight)*Int( VertScrollBar.Range )) );
    END;
  END;
END;

(*----------------------------------------------------*)
procedure TMDIChild.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

BEGIN
bMouseCaptured	:= SetCapture( GetCapture ) <> 0;
IF bMouseCaptured THEN
  BEGIN
  tpMouseCaptured.X := X;
  tpMouseCaptured.Y := Y;
  DisplayMouseCoordinates( tpMouseCaptured );
  END;
END;

(*----------------------------------------------------*)
PROCEDURE TMDIChild.RepositionScrollBars( aPoint: TPoint );

BEGIN
WITH theSubForm.ScrollBox1, aPoint DO
  BEGIN
  HorzScrollBar.Position := Round(Int(X)/Int(PaintBox1.ClientWidth)*Int(HorzScrollBar.Range ))-theSubForm.ClientWidth SHR 1;
  VertScrollBar.Position := Round(Int(Y)/Int(PaintBox1.ClientHeight)*Int( VertScrollBar.Range ))-PaintBox1.ClientHeight;
  END;
END;

(*----------------------------------------------------*)
procedure TMDIChild.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);

VAR
  tpDistance: TPoint;

BEGIN
DisplayMouseCoordinates( Point( X, Y ) );
IF bMouseCaptured THEN
  BEGIN
  tpDistance.X := X-tpMouseCaptured.X;
  tpDistance.Y := Y-tpMouseCaptured.Y;
  IF ((tpDistance.X*tpDistance.X) + (tpDistance.Y*tpDistance.Y)) > 25 THEN
    RepositionScrollBars( Point(X,Y) );
  END;
END;

(*----------------------------------------------------*)
procedure TMDIChild.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

BEGIN
IF bMouseCaptured THEN
  BEGIN
  ReleaseCapture;
  bMouseCaptured	:= FALSE;
  tpMouseCaptured	:= Point( 0, 0 );
  END;
END;

(*----------------------------------------------------*)
procedure TMDIChild.PaintBox1DblClick(Sender: TObject);

BEGIN
RepositionScrollBars( tpLastMousePosition );
END;

(*----------------------------------------------------*)
PROCEDURE TMDIChild.CMParentCalled( VAR aMsg: TMessage );

BEGIN
WITH aMsg DO
  BEGIN
  CASE wParam OF
    cmPCNothing    :;
    cmPCChangeSizeX:
      theSizeLog.X := lParam;
    cmPCChangeSizeY:
      theSizeLog.Y := lParam;
    cmPCNewFileName:
      Caption      := ExtractFileName(theSubForm.Caption);
    END;
  END;
INHERITED;
END;

(*----------------------------------------------------*)
procedure TMDIChild.FormDestroy(Sender: TObject);

begin
theSubForm.Free;
end;

(*++++++++++++++++++++++++++++++++++++++++++++++++++++*)

END.
